@@ -18,13 +18,13 @@
 #  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA #
 #                                                                             #
 #  Contact: Guillaume.Huard@imag.fr                                           #
-#           ENSIMAG - Laboratoire LIG                                         #
+#           ENSIMAG - Laboratoire ID                                          #
 #           51 avenue Jean Kuntzmann                                          #
 #           38330 Montbonnot Saint Martin                                     #
 #                                                                             #
 ###############################################################################
 
-package TakTuk;
+package taktuk;
 
 # Turns autoflush on for the given filehandle
 sub no_flush($);
@@ -64,15 +64,13 @@ sub recv(%);
 sub wait_message(@);
 # Get some info from TakTuk
 sub get($);
- 
-BEGIN
-  {
-    die "FATAL : Perl interpreter too old ($]), Taktuk require Perl >= 5.6.1\n"
-        if ($] < 5.006001);
-  }
+# Decode a string specifying a set of intervals and transforms it to an array
+sub decode_set($);
+# Encode an array of intervals into a compact string specification
+sub encode_set(@);
 
-our $VERSION = "3.6.3";
-our $RELEASE = sprintf "%d", q$Rev: 477 $ =~ /(\d+)/g;
+our $VERSION = "3.6";
+our $RELEASE = sprintf "%d", q$Rev: 439 $ =~ /(\d+)/g;
 # syscalls default granularity
 our $read_size = 32768;
 our $write_size = 32768;
@@ -88,7 +86,6 @@ our $info="I";
 our $option="N";
 our $timeout="O";
 our $put="P";
-our $reduce_result="Q";
 our $reduce="R";
 our $spread="S";
 our $taktuk_code="T";
@@ -126,7 +123,7 @@ our $reduce_tree = 't';
 ### dispatching of connectors execution     ###
 ###############################################
 
-package TakTuk;
+package taktuk;
 use strict; use bytes;
 
 our %buffer;
@@ -267,7 +264,7 @@ sub get_message ($)
 
     if (exists($buffer{$descriptor}))
       {
-        my ($message, $new_buffer) = TakTuk::unpack($buffer{$descriptor});
+        my ($message, $new_buffer) = taktuk::unpack($buffer{$descriptor});
 
         if (defined($new_buffer))
           {
@@ -343,13 +340,13 @@ our $control_channel_write;
 
 if ($ENV{TAKTUK_CONTROL_READ})
   {
-    open($control_channel_read, "<&=". $ENV{TAKTUK_CONTROL_READ})
+    open($control_channel_read, "<&=", $ENV{TAKTUK_CONTROL_READ})
         or print("Error opening taktuk control channel : $!\n");
     binmode($control_channel_read);
   }
 if ($ENV{TAKTUK_CONTROL_WRITE})
   {
-    open($control_channel_write, ">&=". $ENV{TAKTUK_CONTROL_WRITE})
+    open($control_channel_write, ">&=", $ENV{TAKTUK_CONTROL_WRITE})
         or print("Error opening taktuk control channel : $!\n");
     no_flush($control_channel_write);
   }
@@ -365,7 +362,7 @@ use constant EINVAL=>8;
 use constant ENOERR=>9;
 
 our @taktuk_errors = (
-  '"TakTuk::syswrite failed, system message : $!"',
+  '"taktuk::syswrite failed, system message : $!"',
   '"TakTuk engine closed the communication channel"',
   '"sysread error, system message : $!"',
   '"field \"to\" not defined"',
@@ -409,16 +406,26 @@ sub send(%)
       }
     my $body = $argument{body};
 
-    my $full_message = TakTuk::encode($TakTuk::send_to,
-                                  TakTuk::pack($to).
-                                  TakTuk::encode($TakTuk::message,
-                                  TakTuk::pack($to).
-                                  TakTuk::pack($from).
-                                  $body));
-    my $result = TakTuk::syswrite($control_channel_write,
-                                  TakTuk::pack($full_message));
-    $error=ESWRIT if not $result;
-    return $result?$result:undef;
+    my @send_set = taktuk::decode_set($to);
+    if (scalar(@send_set))
+      {
+        $to = taktuk::encode_set(@send_set);
+        my $full_message = taktuk::encode($taktuk::send_to,
+                                      taktuk::pack($to).
+                                      taktuk::encode($taktuk::message,
+                                      taktuk::pack($to).
+                                      taktuk::pack($from).
+                                      $body));
+        my $result = taktuk::syswrite($control_channel_write,
+                                      taktuk::pack($full_message));
+        $error=ESWRIT if not $result;
+        return $result?$result:undef;
+      }
+    else
+      {
+        $error=EINVST;
+        return undef;
+      }
   }
 
 sub recv(%)
@@ -432,13 +439,13 @@ sub recv(%)
     # already there (we have to count waiters internally)
     if (exists($argument{timeout}))
       {
-        $message = TakTuk::encode($TakTuk::wait_message, $argument{timeout});
+        $message = taktuk::encode($taktuk::wait_message, $argument{timeout});
       }
     else
       {
-        $message = $TakTuk::wait_message;
+        $message = $taktuk::wait_message;
       }
-    $result = TakTuk::syswrite($control_channel_write,TakTuk::pack($message));
+    $result = taktuk::syswrite($control_channel_write,taktuk::pack($message));
     if (not $result)
       {
         $error=ESWRIT;
@@ -447,18 +454,18 @@ sub recv(%)
 
     # Now we actually get the message
     my $message_code;
-    ($message_code,$message) = wait_message($TakTuk::timeout,$TakTuk::message);
+    ($message_code,$message) = wait_message($taktuk::timeout,$taktuk::message);
 
     if (defined($message_code))
       {
         my ($to, $from);
-        if ($message_code eq $TakTuk::timeout)
+        if ($message_code eq $taktuk::timeout)
           {
             $error=ETMOUT;
             return ();
           }
-        ($to, $message) = TakTuk::unpack($message);
-        ($from, $message) = TakTuk::unpack($message);
+        ($to, $message) = taktuk::unpack($message);
+        ($from, $message) = taktuk::unpack($message);
 
         return ($to, $from, $message);
       }
@@ -494,7 +501,7 @@ sub wait_message(@)
         $message = get_message($control_channel_read);
         while ($message)
           {
-            ($code, $body) = TakTuk::decode($message);
+            ($code, $body) = taktuk::decode($message);
             foreach my $message_code (@codes)
               {
                 return ($code, $body) if ($message_code eq $code);
@@ -521,8 +528,8 @@ sub get($)
     my $message;
 
     # Infos query
-    $message = TakTuk::encode($TakTuk::get_info, shift);
-    $result = TakTuk::syswrite($control_channel_write,TakTuk::pack($message));
+    $message = taktuk::encode($taktuk::get_info, shift);
+    $result = taktuk::syswrite($control_channel_write,taktuk::pack($message));
     if (not $result)
       {
         $error=ESWRIT;
@@ -531,11 +538,11 @@ sub get($)
 
     # Now we actually get the message
     my $message_code;
-    ($message_code,$message) = wait_message($TakTuk::info,$TakTuk::invalid);
+    ($message_code,$message) = wait_message($taktuk::info,$taktuk::invalid);
 
     if (defined($message_code))
       {
-        if ($message_code eq $TakTuk::invalid)
+        if ($message_code eq $taktuk::invalid)
           {
             $error=EINVAL;
             return -1;
@@ -552,194 +559,84 @@ sub get($)
       }
   }
 
+sub decode_set($)
+  {
+    my $string = shift;
+    my @result = ();
+    my $interval;
+    my $i;
+
+    ($interval,$string) = split /\//,$string,2;
+    while (length($interval))
+      {
+        my ($min, $max) = split /-/,$interval,2;
+        my @replacement = ();
+        my $state = 0;
+        my $length = 0;
+
+        $i = $#result;
+        $max = $min if not $max;
+        return () if ($min !~ /^\d+$/o) or ($max !~ /^\d+$/o) or
+                     ($min > $max);
+        while (($i > -1) and ($max < $result[$i]))
+          {
+            $i--;
+            $state = 1 - $state;
+          }
+        unshift(@replacement, $max) if not $state;
+        while (($i > -1) and ($min <= $result[$i]))
+          {
+            $i--;
+            $state = 1 - $state;
+            $length++;
+          }
+        unshift(@replacement, $min) if not $state;
+        $i++;
+        #print "splice @result, $i, $length, @replacement;\n";
+        splice @result, $i, $length, @replacement;
+
+        if (defined($string))
+          {
+            ($interval,$string) = split /\//,$string,2;
+          }
+        else
+          {
+            $interval = "";
+          }
+      }
+    for ($i=1; $i<$#result; $i+=2)
+      {
+        splice @result, $i, 2
+            while (($i<$#result) and ($result[$i]+1 == $result[$i+1]));
+      }
+    return @result;
+  }
+
+sub encode_set(@)
+  {
+    my @set = @_;
+    my $result = "";
+
+    while (scalar(@set))
+      {
+        my $min = shift @set;
+        my $max = shift @set;
+
+        $result .= "/" if ($result);
+        if ($min == $max)
+          {
+            $result .= "$min";
+          }
+        else
+          {
+            $result .= "$min-$max";
+          }
+      }
+    return $result;
+  }
+
 ###############################################
 ### COMMAND                                 ###
 ###############################################
 
 1;
-###############################################################################
-#                                                                             #
-#  TakTuk, a middleware for adaptive large scale parallel remote executions   #
-#  deployment. Perl implementation, copyright(C) 2006 Guillaume Huard.        #
-#                                                                             #
-#  This program is free software; you can redistribute it and/or modify       #
-#  it under the terms of the GNU General Public License as published by       #
-#  the Free Software Foundation; either version 2 of the License, or          #
-#  (at your option) any later version.                                        #
-#                                                                             #
-#  This program is distributed in the hope that it will be useful,            #
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of             #
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              #
-#  GNU General Public License for more details.                               #
-#                                                                             #
-#  You should have received a copy of the GNU General Public License          #
-#  along with this program; if not, write to the Free Software                #
-#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA #
-#                                                                             #
-#  Contact: Guillaume.Huard@imag.fr                                           #
-#           ENSIMAG - Laboratoire LIG                                         #
-#           51 avenue Jean Kuntzmann                                          #
-#           38330 Montbonnot Saint Martin                                     #
-#                                                                             #
-###############################################################################
-
-=pod TakTuk communication layer interface documentation (Perl interface)
-
-=begin html
-
-<center><h1>USER MANUAL</h1></center>
-
-=end html
-
-=head1 NAME
-
-TakTuk - Perl module that provides an interface to C<taktuk(1)> communication
-facilities
-
-=head1 SYNOPSIS
-
-  use TakTuk;
-  
-  my $rank = TakTuk::get('rank');
-  my $count = TakTuk::get('count');
-  
-  print "I'm process $rank among $count\n";
-  
-  if ($rank > 1) {
-      my ($to, $from, $message) = TakTuk::recv();
-      if (not defined($message)) {
-          print "Trying to recv: ",
-                TakTuk::error_msg($TakTuk::error), "\n";
-      } else {
-          print "$to received $message from $from\n";
-      }
-  }
-  
-  sleep 1;
-  my $next = $rank+1;
-  $next = 1 if ($next > $count);
-  if (not TakTuk::send(to=>$next, body=>"[Salut numero $rank]")) {
-      print "Trying to send to $next: ",
-            TakTuk::error_msg($TakTuk::error), "\n";
-  }
-  
-  if ($rank == 1) {
-      my ($to, $from, $message) = TakTuk::recv(timeout=>5);
-      if (not defined($message)) {
-          print "Trying to recv :",
-                TakTuk::error_msg($TakTuk::error), "\n";
-      } else {
-          print "$to received $message from $from\n";
-      }
-  }
-
-=head1 DESCRIPTION
-
-The B<TakTuk> communication layer Perl interface provides a way for programs
-executed using the C<taktuk(1)> command to exchange data. It is based on a
-simple send/receive model using multicast-like sends and optionally timeouted
-receives.  This is only designed to be a control facility, in particular this
-is not a high performance communication library.
-
-WARNING: the B<TakTuk> communication interface is not process-safe : it is
-probably a very bad idea to use point-to-point communication in more than one
-process related to a single B<TakTuk> instance (these processes might be
-several local commands or forked processes).
-
-The Perl communication interface for B<TakTuk> is made of functions that can be
-called by scripts executed using the C<taktuk_perl> command of the B<TakTuk>
-engine (prefered way, less installation requirements on remote machines) or
-using the B<TakTuk> Perl module provided with the B<TakTuk> distribution.
-These functions are:
-
-=over
-
-=item B<TakTuk::get($)>
-
-gets some information from B<TakTuk>. Currently available informations are
-'rank', 'count', 'father', 'child_min' and 'child_max". This is a better way to
-get these informations than environment variables as its takes into account
-renumbering that might occur after process spawn.
-
-=item B<TakTuk::send(%)>
-
-sends a scalar to a single peer or a set specification (see C<taktuk(1)> for
-informations about set specifications).
-The two mandatory fields in the arguments are C<to> (with a set specification)
-and C<body>. Returns an undefined value upon error.
-
-=item B<TakTuk::recv(%)>
-
-blocks until the reception of a message. Returns a list of three elements: the
-logical number of the destination of the message, the logical number of its
-source and the message itself.
-Accepts an optional C<timeout> argument with a numeric value.
-Returns an empty list upon error.
-
-=back
-
-When an error occur, both of these functions set the variable C<$TakTuk::error>
-to the numeric code of the error that occured. A textual description of the
-error is provided by the function C<TakTuk::error_msg($)> that takes the error
-code as an argument.
-
-Error codes are the following :
-
-=over
-
-=item TakTuk::ESWRIT
-
-a call to C<TakTuk::syswrite> failed. This is due to a C<syswrite> error
-different than C<EAGAIN>. The code should be accessible using C<$!>.
-
-=item TakTuk::EFCLSD
-
-the communication channel to the B<TakTuk> engine has been closed. This
-typically occur when shutting down the logical network (using Ctrl-C on root
-node for instance).
-
-=item TakTuk::ESREAD (C<TakTuk::recv> only)
-
-a call to C<sysread> failed (the code should be accessible using C<$!>).
-
-=item TakTuk::EARGTO (C<TakTuk::send> only)
-
-C<to> field missing in the arguments.
-
-=item TakTuk::EARGBD (C<TakTuk::send> only)
-
-C<body> field missing in the arguments.
-
-=item TakTuk::ETMOUT (C<TakTuk::recv> only)
-
-The call to C<TakTuk::recv> timeouted. This only occur when giving a C<timeout>
-field as C<TakTuk::recv> argument.
-
-=begin comment
-
-=item TakTuk::EINVST (C<TakTuk::send> only)
-
-The set specification given as a destination to the C<TakTuk::send> function is
-not correct.
-
-=end comment
-
-=back
-
-=head1 SEE ALSO
-
-C<tatkuk(1)>, C<taktukcomm(3)>, C<TakTuk::Pilot(3)>
-
-=head1 AUTHOR
-
-The original concept of B<TakTuk> has been proposed by Cyrille Martin in his PhD thesis. People involved in this work include Jacques Briat, Olivier Richard, Thierry Gautier and Guillaume Huard.
-
-The author of the version 3 (perl version) and current maintainer of the package is Guillaume Huard.
-
-=head1 COPYRIGHT
-
-The C<TakTuk> communication interface library is provided under the terms
-of the GNU General Public License version 2 or later.
-
-=cut
-
